==> Alpha_Rabbit-1.0.7-py3-none-any.whl: Alpha_Rabbit/Factor_Calculator.py <==
from Alpha_Rabbit import *
_method = Factor_get_method()
def Factor_Calculator(pricebyday,minbar,conn,todaydate,notst,factors_to_cal):
    ######################################## 日间数据计算因子 ####################################
    uploadfactordict = {}
    Close = pricebyday['close'].unstack()
    Open = pricebyday['open'].unstack()
    High = pricebyday['high'].unstack()
    Low = pricebyday['low'].unstack()
    volume = pricebyday[['volume']].pivot_table(index = 'date',columns = 'symbol',values = 'volume')
    total_turnover = pricebyday[['total_turnover']].pivot_table(index = 'date',columns = 'symbol',values = 'total_turnover')



    tempClose = Close.iloc[-30:]
    tempOpen = Open.iloc[-30:]
    tempHigh = High.iloc[-30:]
    tempLow = Low.iloc[-30:]
    anaual_close = Close.iloc[-272:]
    anaual_High = High.iloc[-272:]
    
    if 'mmt_intraday_M' in factors_to_cal or factors_to_cal == 'all':
        # 1个月日内动量
        uploadfactordict['mmt_intraday_M'] = mmt_intraday_M(tempClose,tempOpen)
    if 'mmt_range_M' in factors_to_cal or factors_to_cal == 'all':
        # 一个月振幅调整动量
        uploadfactordict['mmt_range_M'] = mmt_range_M(tempHigh,tempLow,tempClose)
    if 'mmt_overnight_M' in factors_to_cal or factors_to_cal == 'all':
        # 隔夜动量
        uploadfactordict['mmt_overnight_M'] = mmt_overnight_M(tempOpen,tempClose)
    if 'mmt_route_M' in factors_to_cal or factors_to_cal == 'all':
        # 路径调整动量
        uploadfactordict['mmt_route_M'] = mmt_route_M(tempClose)
    if 'mmt_discrete_M' in factors_to_cal or factors_to_cal == 'all':
        # 信息离散度动量
        uploadfactordict['mmt_discrete_M'] = mmt_discrete_M(tempClose)
    if 'mmt_sec_rank_M' in factors_to_cal or factors_to_cal == 'all':
        # 截面rank动量
        uploadfactordict['mmt_sec_rank_M'] = mmt_sec_rank_M(tempClose)
    if 'mmt_time_rank_M' in factors_to_cal or factors_to_cal == 'all':
        # 时序rank_score
        uploadfactordict['mmt_time_rank_M'] = mmt_time_rank_M(anaual_close)
    if 'mmt_highest_days_A' in factors_to_cal or factors_to_cal == 'all':
        # 最高价距今天数
        uploadfactordict['mmt_highest_days_A'] = mmt_highest_days_A(anaual_High)
    if 'volumestable' in factors_to_cal or factors_to_cal == 'all':
        # 成交量稳定度
        uploadfactordict['volumestable'] = volumestable(volume)
    if '_con' in factors_to_cal or factors_to_cal == 'all':
        # 收益一致性因子
        uploadfactordict['_con'] = re_con(tempClose)
    if 'bofu_money' in factors_to_cal or factors_to_cal == 'all':
        # 波幅/成交额
        uploadfactordict['bofu_money'] = bofu_money(tempHigh,tempLow,tempOpen,total_turnover)


    ######################################## 日内数据计算因子 ####################################

    # #单笔成交金额相关因子
    sing_trade_amt = pd.DataFrame(minbar['total_turnover']/minbar['num_trades'],columns= ['single_trade_amt'])
    sing_trade_amt = sing_trade_amt[sing_trade_amt['single_trade_amt']>=0]
    sing_trade_amt['trading_date'] = todaydate
    sta_del_extrm = sing_trade_amt.groupby(level = 0).apply(lambda x: x.sort_values('single_trade_amt').iloc[:-10]).droplevel(0)# 剔除极大值
    sta_50pct = sing_trade_amt.groupby(level = 0).\
        apply(lambda x: x[x['single_trade_amt']<x['single_trade_amt'].quantile(0.5)]).droplevel(0)# 后百分之五十


    if 'mts' in factors_to_cal or factors_to_cal == 'all':
        # 主力交易强度
        uploadfactordict['mts'] = mts(sta_del_extrm,minbar,todaydate)

    if 'mte' in factors_to_cal or factors_to_cal == 'all':
        # 主力交易情绪
        uploadfactordict['mte'] = mte(sta_del_extrm,minbar,todaydate)

    if 'qua' in factors_to_cal or factors_to_cal == 'all':
        # 分位数因子qua
        uploadfactordict['qua'] = qua(sta_del_extrm,todaydate)

    if 'qua20m' in factors_to_cal or factors_to_cal == 'all':
        prv_factor = _method.get_prev_days_factor_by_name('qua',notst.index[-20],conn)
        q = qua(sta_del_extrm,todaydate)
        qua20m = prv_factor.append(q).unstack().rolling(20).mean().iloc[-1:].stack().rename(columns = {'qua':'qua20m'})
        uploadfactordict['qua20m'] = qua20m

    if 'skew' in factors_to_cal or factors_to_cal == 'all':
        # 偏度因子skew
        uploadfactordict['skew'] = skew(sta_50pct,todaydate)

    if 'skew20m' in factors_to_cal or factors_to_cal == 'all':
        prv_factor = _method.get_prev_days_factor_by_name('skew',notst.index[-20],conn)
        sk = skew(sta_50pct,todaydate)
        skew20m = prv_factor.append(sk).unstack().rolling(20).mean().iloc[-1:].stack().rename(columns = {'skew':'skew20m'})
        uploadfactordict['skew20m'] = skew20m

    if 's_reverse' in factors_to_cal or factors_to_cal == 'all':
        # 强反转因子
        uploadfactordict['s_reverse'] = s_reverse(sing_trade_amt,minbar,todaydate)

    if 's_reverse_10_sum' in factors_to_cal or factors_to_cal == 'all':
        prv_factor = _method.get_prev_days_factor_by_name('s_reverse',notst.index[-10],conn)
        sr = s_reverse(sing_trade_amt,minbar,todaydate)
        s_reverse_10_sum = prv_factor.append(sr).unstack().rolling(10).sum().iloc[-1:].stack().rename(columns = {'s_reverse':'s_reverse_10_sum'})
        uploadfactordict['s_reverse_10_sum'] = s_reverse_10_sum

    if 'daily_sta_90pct' in factors_to_cal or factors_to_cal == 'all':
        # 理想反转因子
        uploadfactordict['daily_sta_90pct'] = daily_sta_90pct(sta_del_extrm)
    
    if 'ideal_reverse' in factors_to_cal or factors_to_cal == 'all':
        prv_factor = _method.get_prev_days_factor_by_name('daily_sta_90pct',notst.index[-20],conn)
        dsta90 = daily_sta_90pct(sing_trade_amt)
        daily_sta_cal = prv_factor.append(dsta90)
        uploadfactordict['ideal_reverse'] = ideal_reverse(daily_sta_cal,Close)

    return uploadfactordict

==> Alpha_Rabbit-1.1.2-py3-none-any.whl: Alpha_Rabbit/Factor_Calculator.py <==
from Alpha_Rabbit.Factor_Def_and_Get_Method import *
_method = Factor_get_method()
def Factor_Calculator(pricebyday,minbar,conn,todaydate,notst,factors_to_cal):
    ######################################## 日间数据计算因子 ####################################
    uploadfactordict = {}
    Close = pricebyday['close'].unstack().sort_index()
    Open = pricebyday['open'].unstack().sort_index()
    High = pricebyday['high'].unstack().sort_index()
    Low = pricebyday['low'].unstack().sort_index()
    volume = pricebyday[['volume']].pivot_table(index = 'date',columns = 'symbol',values = 'volume').sort_index()
    total_turnover = pricebyday[['total_turnover']].pivot_table(index = 'date',columns = 'symbol',values = 'total_turnover').sort_index()



    tempClose = Close.iloc[-30:]
    tempOpen = Open.iloc[-30:]
    tempHigh = High.iloc[-30:]
    tempLow = Low.iloc[-30:]
    anaual_close = Close.iloc[-272:]
    anaual_High = High.iloc[-272:]
    
    if 'mmt_intraday_M' in factors_to_cal or factors_to_cal == 'all':
        # 1个月日内动量
        uploadfactordict['mmt_intraday_M'] = mmt_intraday_M(tempClose,tempOpen)
    if 'mmt_range_M' in factors_to_cal or factors_to_cal == 'all':
        # 一个月振幅调整动量
        uploadfactordict['mmt_range_M'] = mmt_range_M(tempHigh,tempLow,tempClose)
    if 'mmt_overnight_M' in factors_to_cal or factors_to_cal == 'all':
        # 隔夜动量
        uploadfactordict['mmt_overnight_M'] = mmt_overnight_M(tempOpen,tempClose)
    if 'mmt_route_M' in factors_to_cal or factors_to_cal == 'all':
        # 路径调整动量
        uploadfactordict['mmt_route_M'] = mmt_route_M(tempClose)
    if 'mmt_discrete_M' in factors_to_cal or factors_to_cal == 'all':
        # 信息离散度动量
        uploadfactordict['mmt_discrete_M'] = mmt_discrete_M(tempClose)
    if 'mmt_sec_rank_M' in factors_to_cal or factors_to_cal == 'all':
        # 截面rank动量
        uploadfactordict['mmt_sec_rank_M'] = mmt_sec_rank_M(tempClose)
    if 'mmt_time_rank_M' in factors_to_cal or factors_to_cal == 'all':
        # 时序rank_score
        uploadfactordict['mmt_time_rank_M'] = mmt_time_rank_M(anaual_close)
    if 'mmt_highest_days_A' in factors_to_cal or factors_to_cal == 'all':
        # 最高价距今天数
        uploadfactordict['mmt_highest_days_A'] = mmt_highest_days_A(anaual_High)
    if 'volumestable' in factors_to_cal or factors_to_cal == 'all':
        # 成交量稳定度
        uploadfactordict['volumestable'] = volumestable(volume)
    if '_con' in factors_to_cal or factors_to_cal == 'all':
        # 收益一致性因子
        uploadfactordict['_con'] = re_con(tempClose)
    if 'bofu_money' in factors_to_cal or factors_to_cal == 'all':
        # 波幅/成交额
        uploadfactordict['bofu_money'] = bofu_money(tempHigh,tempLow,tempOpen,total_turnover)


    ######################################## 日内数据计算因子 ####################################

    # #单笔成交金额相关因子
    sing_trade_amt = pd.DataFrame(minbar['total_turnover']/minbar['num_trades'],columns= ['single_trade_amt'])
    sing_trade_amt = sing_trade_amt[sing_trade_amt['single_trade_amt']>=0]
    sing_trade_amt['trading_date'] = todaydate
    sta_del_extrm = sing_trade_amt.groupby(level = 0).apply(lambda x: x.sort_values('single_trade_amt').iloc[:-10]).droplevel(0)# 剔除极大值
    sta_50pct = sing_trade_amt.groupby(level = 0).\
        apply(lambda x: x[x['single_trade_amt']<x['single_trade_amt'].quantile(0.5)]).droplevel(0)# 后百分之五十


    if 'mts' in factors_to_cal or factors_to_cal == 'all':
        # 主力交易强度
        uploadfactordict['mts'] = mts(sta_del_extrm,minbar,todaydate)

    if 'mte' in factors_to_cal or factors_to_cal == 'all':
        # 主力交易情绪
        uploadfactordict['mte'] = mte(sta_del_extrm,minbar,todaydate)

    if 'qua' in factors_to_cal or factors_to_cal == 'all':
        # 分位数因子qua
        uploadfactordict['qua'] = qua(sta_del_extrm,todaydate)

    if 'qua20m' in factors_to_cal or factors_to_cal == 'all':
        prv_factor = _method.get_prev_days_factor_by_name('qua',notst.index[-20],conn)
        q = qua(sta_del_extrm,todaydate)
        qua20m = prv_factor.append(q).unstack().rolling(20).mean().iloc[-1:].stack().rename(columns = {'qua':'qua20m'})
        uploadfactordict['qua20m'] = qua20m

    if 'skew' in factors_to_cal or factors_to_cal == 'all':
        # 偏度因子skew
        uploadfactordict['skew'] = skew(sta_50pct,todaydate)

    if 'skew20m' in factors_to_cal or factors_to_cal == 'all':
        prv_factor = _method.get_prev_days_factor_by_name('skew',notst.index[-20],conn)
        sk = skew(sta_50pct,todaydate)
        skew20m = prv_factor.append(sk).unstack().rolling(20).mean().iloc[-1:].stack().rename(columns = {'skew':'skew20m'})
        uploadfactordict['skew20m'] = skew20m

    if 's_reverse' in factors_to_cal or factors_to_cal == 'all':
        # 强反转因子
        uploadfactordict['s_reverse'] = s_reverse(sing_trade_amt,minbar,todaydate)

    if 's_reverse_10_sum' in factors_to_cal or factors_to_cal == 'all':
        prv_factor = _method.get_prev_days_factor_by_name('s_reverse',notst.index[-10],conn)
        sr = s_reverse(sing_trade_amt,minbar,todaydate)
        s_reverse_10_sum = prv_factor.append(sr).unstack().rolling(10).sum().iloc[-1:].stack().rename(columns = {'s_reverse':'s_reverse_10_sum'})
        uploadfactordict['s_reverse_10_sum'] = s_reverse_10_sum

    if 'daily_sta_90pct' in factors_to_cal or factors_to_cal == 'all':
        # 理想反转因子
        uploadfactordict['daily_sta_90pct'] = daily_sta_90pct(sta_del_extrm)
    
    if 'ideal_reverse' in factors_to_cal or factors_to_cal == 'all':
        prv_factor = _method.get_prev_days_factor_by_name('daily_sta_90pct',notst.index[-20],conn)
        dsta90 = daily_sta_90pct(sing_trade_amt)
        daily_sta_cal = prv_factor.append(dsta90)
        uploadfactordict['ideal_reverse'] = ideal_reverse(daily_sta_cal,Close)

    return uploadfactordict
